@@ -1,9 +1,7 @@
 //
 // SQLiteStatementImpl.cpp
 //
-// $Id: //poco/Main/Data/SQLite/src/SQLiteStatementImpl.cpp#8 $
-//
-// Library: SQLite
+// Library: Data/SQLite
 // Package: SQLite
 // Module:  SQLiteStatementImpl
 //
@@ -20,11 +18,7 @@
 #include "Poco/String.h"
 #include <cstdlib>
 #include <cstring>
-#if defined(POCO_UNBUNDLED)
 #include <sqlite3.h>
-#else
-#include "sqlite3.h"
-#endif
 
 
 namespace Poco {
@@ -38,7 +32,7 @@ const int SQLiteStatementImpl::POCO_SQLITE_INV_ROW_CNT = -1;
 SQLiteStatementImpl::SQLiteStatementImpl(Poco::Data::SessionImpl& rSession, sqlite3* pDB):
 	StatementImpl(rSession),
 	_pDB(pDB),
-	_pStmt(0),
+	_pStmt(nullptr),
 	_stepCalled(false),
 	_nextResponse(0),
 	_affectedRowCount(POCO_SQLITE_INV_ROW_CNT),
@@ -48,7 +42,7 @@ SQLiteStatementImpl::SQLiteStatementImpl(Poco::Data::SessionImpl& rSession, sqli
 {
 	_columns.resize(1);
 }
-	
+
 
 SQLiteStatementImpl::~SQLiteStatementImpl()
 {
@@ -72,14 +66,14 @@ void SQLiteStatementImpl::compileImpl()
 
 	std::string statement(toString());
 
-	sqlite3_stmt* pStmt = 0;
+	sqlite3_stmt* pStmt = nullptr;
 	const char* pSql = _pLeftover ? _pLeftover->c_str() : statement.c_str();
 
 	if (0 == std::strlen(pSql))
 		throw InvalidSQLStatementException("Empty statements are illegal");
 
 	int rc = SQLITE_OK;
-	const char* pLeftover = 0;
+	const char* pLeftover = nullptr;
 	bool queryFound = false;
 
 	do
@@ -88,9 +82,9 @@ void SQLiteStatementImpl::compileImpl()
 		if (rc != SQLITE_OK)
 		{
 			if (pStmt) sqlite3_finalize(pStmt);
-			pStmt = 0;
+			pStmt = nullptr;
 			std::string errMsg = sqlite3_errmsg(_pDB);
-			Utility::throwException(rc, errMsg);
+			Utility::throwException(_pDB, rc, errMsg);
 		}
 		else if (rc == SQLITE_OK && pStmt)
 		{
@@ -155,18 +149,18 @@ void SQLiteStatementImpl::bindImpl()
 {
 	_stepCalled = false;
 	_nextResponse = 0;
-	if (_pStmt == 0) return;
+	if (_pStmt == nullptr) return;
 
 	sqlite3_reset(_pStmt);
 
-	int paramCount = sqlite3_bind_parameter_count(_pStmt);
-	BindIt bindEnd = bindings().end();
-	if (0 == paramCount || bindEnd == _bindBegin) 
+	std::size_t paramCount = static_cast<std::size_t>(sqlite3_bind_parameter_count(_pStmt));
+	if (0 == paramCount)
 	{
 		_canBind = false;
 		return;
 	}
 
+	BindIt bindEnd = bindings().end();
 	std::size_t availableCount = 0;
 	Bindings::difference_type bindCount = 0;
 	Bindings::iterator it = _bindBegin;
@@ -177,6 +171,9 @@ void SQLiteStatementImpl::bindImpl()
 		else break;
 	}
 
+	if (availableCount < paramCount)
+		throw ParameterCountMismatchException();
+
 	Bindings::difference_type remainingBindCount = bindEnd - _bindBegin;
 	if (bindCount < remainingBindCount)
 	{
@@ -220,9 +217,9 @@ void SQLiteStatementImpl::clear()
 	if (_pStmt)
 	{
 		sqlite3_finalize(_pStmt);
-		_pStmt=0;
+		_pStmt=nullptr;
 	}
-	_pLeftover = 0;
+	_pLeftover = nullptr;
 }
 
 
@@ -232,7 +229,7 @@ bool SQLiteStatementImpl::hasNext()
 		return (_nextResponse == SQLITE_ROW);
 
 	// _pStmt is allowed to be null for conditional SQL statements
-	if (_pStmt == 0)
+	if (_pStmt == nullptr)
 	{
 		_stepCalled   = true;
 		_nextResponse = SQLITE_DONE;
@@ -247,7 +244,7 @@ bool SQLiteStatementImpl::hasNext()
 		_affectedRowCount += sqlite3_changes(_pDB);
 
 	if (_nextResponse != SQLITE_ROW && _nextResponse != SQLITE_OK && _nextResponse != SQLITE_DONE)
-		Utility::throwException(_nextResponse);
+		Utility::throwException(_pDB, _nextResponse);
 
 	_pExtractor->reset();//clear the cached null indicators
 
@@ -259,7 +256,7 @@ std::size_t SQLiteStatementImpl::next()
 {
 	if (SQLITE_ROW == _nextResponse)
 	{
-		poco_assert (columnsReturned() == sqlite3_column_count(_pStmt));
+		poco_assert (columnsReturned() == static_cast<std::size_t>(sqlite3_column_count(_pStmt)));
 
 		Extractions& extracts = extractions();
 		Extractions::iterator it    = extracts.begin();
@@ -273,7 +270,10 @@ std::size_t SQLiteStatementImpl::next()
 		}
 		_stepCalled = false;
 		if (_affectedRowCount == POCO_SQLITE_INV_ROW_CNT) _affectedRowCount = 0;
-		_affectedRowCount += (*extracts.begin())->numOfRowsHandled();
+		if (extracts.begin() != extracts.end())
+		{
+			_affectedRowCount += static_cast<int>((*extracts.begin())->numOfRowsHandled());
+		}
 	}
 	else if (SQLITE_DONE == _nextResponse)
 	{
@@ -281,9 +281,9 @@ std::size_t SQLiteStatementImpl::next()
 	}
 	else
 	{
-		Utility::throwException(_nextResponse, std::string("Iterator Error: trying to access the next value"));
+		Utility::throwException(_pDB, _nextResponse, std::string("Iterator Error: trying to access the next value"));
 	}
-	
+
 	return 1u;
 }
 
@@ -297,7 +297,7 @@ std::size_t SQLiteStatementImpl::columnsReturned() const
 const MetaColumn& SQLiteStatementImpl::metaColumn(std::size_t pos) const
 {
 	std::size_t curDataSet = currentDataSet();
-	poco_assert (pos >= 0 && pos <= _columns[curDataSet].size());
+	poco_assert (pos <= _columns[curDataSet].size());
 	return _columns[curDataSet][pos];
 }
 
@@ -305,7 +305,7 @@ const MetaColumn& SQLiteStatementImpl::metaColumn(std::size_t pos) const
 int SQLiteStatementImpl::affectedRowCount() const
 {
 	if (_affectedRowCount != POCO_SQLITE_INV_ROW_CNT) return _affectedRowCount;
-	return _pStmt == 0 || sqlite3_stmt_readonly(_pStmt) ? 0 : sqlite3_changes(_pDB);
+	return _pStmt == nullptr || sqlite3_stmt_readonly(_pStmt) ? 0 : sqlite3_changes(_pDB);
 }
 
 
